@@ -206,6 +206,9 @@ static inline void ArrayLoopHash(Vector &input, Vector &hashes, const SelectionV
 		for (idx_t i = 0; i < count; i++) {
 			auto lidx = idata.sel->get_index(i);
 			if (idata.validity.RowIsValid(lidx)) {
+				if (FIRST_HASH) {
+					hdata[i] = 0;
+				}
 				for (idx_t j = 0; j < array_size; j++) {
 					auto offset = lidx * array_size + j;
 					hdata[i] = CombineHashScalar(hdata[i], chdata[offset]);
@@ -233,6 +236,9 @@ static inline void ArrayLoopHash(Vector &input, Vector &hashes, const SelectionV
 				VectorOperations::Hash(dict_vec, array_hashes, array_size);
 				auto ahdata = FlatVector::GetData<hash_t>(array_hashes);
 
+				if (FIRST_HASH) {
+					hdata[ridx] = 0;
+				}
 				// Combine the hashes of the array
 				for (idx_t j = 0; j < array_size; j++) {
 					hdata[ridx] = CombineHashScalar(hdata[ridx], ahdata[j]);
