@@ -2,28 +2,15 @@
 
 namespace duckdb {
 
-CSVPosition::CSVPosition(idx_t file_idx_p, idx_t buffer_idx_p, idx_t buffer_pos_p)
-    : file_idx(file_idx_p), buffer_idx(buffer_idx_p), buffer_pos(buffer_pos_p) {
+CSVPosition::CSVPosition(idx_t buffer_idx_p, idx_t buffer_pos_p) : buffer_idx(buffer_idx_p), buffer_pos(buffer_pos_p) {
 }
 CSVPosition::CSVPosition() {
 }
 
-CSVBoundary::CSVBoundary(idx_t file_idx_p, idx_t buffer_idx_p, idx_t buffer_pos_p, idx_t boundary_idx_p,
-                         idx_t end_pos_p)
-    : file_idx(file_idx_p), buffer_idx(buffer_idx_p), buffer_pos(buffer_pos_p), boundary_idx(boundary_idx_p),
-      end_pos(end_pos_p) {
+CSVBoundary::CSVBoundary(idx_t buffer_idx_p, idx_t buffer_pos_p, idx_t boundary_idx_p, idx_t end_pos_p)
+    : buffer_idx(buffer_idx_p), buffer_pos(buffer_pos_p), boundary_idx(boundary_idx_p), end_pos(end_pos_p) {
 }
-CSVBoundary::CSVBoundary()
-    : file_idx(0), buffer_idx(0), buffer_pos(0), boundary_idx(0), end_pos(NumericLimits<idx_t>::Maximum()) {
-}
-CSVIterator::CSVIterator(idx_t file_idx, idx_t buffer_idx, idx_t buffer_pos, idx_t boundary_idx, idx_t buffer_size)
-    : pos(file_idx, buffer_idx, buffer_pos), is_set(true) {
-	// The end of our boundary will be the buffer size itself it that's smaller than where we want to go
-	if (buffer_size < buffer_pos + BYTES_PER_THREAD) {
-		boundary = {file_idx, buffer_idx, buffer_pos, boundary_idx, buffer_size};
-	} else {
-		boundary = {file_idx, buffer_idx, buffer_pos, boundary_idx, buffer_pos + BYTES_PER_THREAD};
-	}
+CSVBoundary::CSVBoundary() : buffer_idx(0), buffer_pos(0), boundary_idx(0), end_pos(NumericLimits<idx_t>::Maximum()) {
 }
 
 CSVIterator::CSVIterator() : is_set(false) {
@@ -32,7 +19,6 @@ CSVIterator::CSVIterator() : is_set(false) {
 void CSVBoundary::Print() {
 #ifndef DUCKDB_DISABLE_PRINT
 	std::cout << "---Boundary: " << boundary_idx << " ---" << '\n';
-	std::cout << "File Index:: " << file_idx << '\n';
 	std::cout << "Buffer Index: " << buffer_idx << '\n';
 	std::cout << "Buffer Pos: " << buffer_pos << '\n';
 	std::cout << "End Pos: " << end_pos << '\n';
@@ -51,6 +37,8 @@ bool CSVIterator::Next(CSVBufferManager &buffer_manager) {
 	if (!is_set) {
 		return false;
 	}
+	// If we are calling next this is not the first one anymore
+	first_one = false;
 	boundary.boundary_idx++;
 	// This is our start buffer
 	auto buffer = buffer_manager.GetBuffer(boundary.buffer_idx);
@@ -84,12 +72,8 @@ idx_t CSVIterator::GetEndPos() const {
 	return boundary.end_pos;
 }
 
-idx_t CSVIterator::GetFileIdx() const {
-	return pos.file_idx;
-}
-
 idx_t CSVIterator::GetBufferIdx() const {
-	return boundary.buffer_idx;
+	return pos.buffer_idx;
 }
 
 idx_t CSVIterator::GetBoundaryIdx() const {
@@ -97,11 +81,27 @@ idx_t CSVIterator::GetBoundaryIdx() const {
 }
 
 void CSVIterator::SetCurrentPositionToBoundary() {
-	pos.file_idx = boundary.file_idx;
 	pos.buffer_idx = boundary.buffer_idx;
 	pos.buffer_pos = boundary.buffer_pos;
 }
 
+void CSVIterator::SetCurrentBoundaryToPosition(bool single_threaded) {
+	if (single_threaded) {
+		is_set = false;
+		return;
+	}
+	boundary.buffer_idx = pos.buffer_idx;
+	if (pos.buffer_pos == 0) {
+		boundary.end_pos = CSVIterator::BYTES_PER_THREAD;
+	} else {
+		boundary.end_pos = ((pos.buffer_pos + CSVIterator::BYTES_PER_THREAD - 1) / CSVIterator::BYTES_PER_THREAD) *
+		                   CSVIterator::BYTES_PER_THREAD;
+	}
+
+	boundary.buffer_pos = boundary.end_pos - CSVIterator::BYTES_PER_THREAD;
+	is_set = true;
+}
+
 void CSVIterator::SetStart(idx_t start) {
 	boundary.buffer_pos = start;
 }
