@@ -6,7 +6,6 @@
 #include "utf8proc.hpp"
 
 namespace duckdb {
-
 // Helper function to generate column names
 static string GenerateColumnName(const idx_t total_cols, const idx_t col_number, const string &prefix = "column") {
 	auto max_digits = NumericHelper::UnsignedLength(total_cols - 1);
@@ -98,26 +97,33 @@ static string NormalizeColumnName(const string &col_name) {
 }
 
 // If our columns were set by the user, we verify if their names match with the first row
-bool CSVSniffer::DetectHeaderWithSetColumn() {
+bool CSVSniffer::DetectHeaderWithSetColumn(ClientContext &context, vector<HeaderValue> &best_header_row,
+                                           SetColumns &set_columns, CSVReaderOptions &options) {
 	bool has_header = true;
 	bool all_varchar = true;
 	bool first_row_consistent = true;
+	std::ostringstream error;
 	// User set the names, we must check if they match the first row
 	// We do a +1 to check for situations where the csv file has an extra all null column
 	if (set_columns.Size() != best_header_row.size() && set_columns.Size() + 1 != best_header_row.size()) {
 		return false;
-	} else {
-		// Let's do a match-aroo
-		for (idx_t i = 0; i < set_columns.Size(); i++) {
-			if (best_header_row[i].IsNull()) {
-				return false;
-			}
-			if (best_header_row[i].value.GetString() != (*set_columns.names)[i]) {
-				has_header = false;
-				break;
-			}
+	}
+
+	// Let's do a match-aroo
+	for (idx_t i = 0; i < set_columns.Size(); i++) {
+		if (best_header_row[i].IsNull()) {
+			return false;
+		}
+		if (best_header_row[i].value != (*set_columns.names)[i]) {
+			error << "Header Mismatch at position:" << i << "\n";
+			error << "Expected Name: \"" << (*set_columns.names)[i] << "\".";
+			error << "Actual Name: \"" << best_header_row[i].value << "\"."
+			      << "\n";
+			has_header = false;
+			break;
 		}
 	}
+
 	if (!has_header) {
 		// We verify if the types are consistent
 		for (idx_t col = 0; col < set_columns.Size(); col++) {
@@ -125,32 +131,60 @@ bool CSVSniffer::DetectHeaderWithSetColumn() {
 			const auto &sql_type = (*set_columns.types)[col];
 			if (sql_type != LogicalType::VARCHAR) {
 				all_varchar = false;
-				if (!CanYouCastIt(best_header_row[col].value, sql_type, options.dialect_options,
-				                  best_header_row[col].IsNull(), options.decimal_separator[0])) {
+				if (!CSVSniffer::CanYouCastIt(context, best_header_row[col].value, sql_type, options.dialect_options,
+				                              best_header_row[col].IsNull(), options.decimal_separator[0])) {
 					first_row_consistent = false;
 				}
 			}
 		}
+		if (!first_row_consistent) {
+			options.sniffer_user_mismatch_error += error.str();
+		}
 		if (all_varchar) {
-			// Can't be the header
-			return false;
+			return true;
 		}
 		return !first_row_consistent;
 	}
 	return has_header;
 }
-void CSVSniffer::DetectHeader() {
-	auto &sniffer_state_machine = best_candidate->GetStateMachine();
+
+bool EmptyHeader(const string &col_name, bool is_null, bool normalize) {
+	if (col_name.empty() || is_null) {
+		return true;
+	}
+	if (normalize) {
+		// normalize has special logic to trim white spaces and generate names
+		return false;
+	}
+	// check if it's all white spaces
+	for (auto &c : col_name) {
+		if (!StringUtil::CharacterIsSpace(c)) {
+			return false;
+		}
+	}
+	// if we are not normalizing the name and is all white spaces, then we generate a name
+	return true;
+}
+
+vector<string>
+CSVSniffer::DetectHeaderInternal(ClientContext &context, vector<HeaderValue> &best_header_row,
+                                 CSVStateMachine &state_machine, SetColumns &set_columns,
+                                 unordered_map<idx_t, vector<LogicalType>> &best_sql_types_candidates_per_column_idx,
+                                 CSVReaderOptions &options, CSVErrorHandler &error_handler) {
+	vector<string> detected_names;
+	auto &dialect_options = state_machine.dialect_options;
 	if (best_header_row.empty()) {
-		sniffer_state_machine.dialect_options.header = false;
-		for (idx_t col = 0; col < sniffer_state_machine.dialect_options.num_cols; col++) {
-			names.push_back(GenerateColumnName(sniffer_state_machine.dialect_options.num_cols, col));
+		dialect_options.header = false;
+		for (idx_t col = 0; col < dialect_options.num_cols; col++) {
+			detected_names.push_back(GenerateColumnName(dialect_options.num_cols, col));
 		}
 		// If the user provided names, we must replace our header with the user provided names
-		for (idx_t i = 0; i < MinValue<idx_t>(names.size(), sniffer_state_machine.options.name_list.size()); i++) {
-			names[i] = sniffer_state_machine.options.name_list[i];
+		if (!options.columns_set) {
+			for (idx_t i = 0; i < MinValue<idx_t>(best_header_row.size(), options.name_list.size()); i++) {
+				detected_names[i] = options.name_list[i];
+			}
 		}
-		return;
+		return detected_names;
 	}
 	// information for header detection
 	bool first_row_consistent = true;
@@ -158,16 +192,17 @@ void CSVSniffer::DetectHeader() {
 	bool first_row_nulls = true;
 	// If null-padding is not allowed and there is a mismatch between our header candidate and the number of columns
 	// We can't detect the dialect/type options properly
-	if (!sniffer_state_machine.options.null_padding &&
-	    best_sql_types_candidates_per_column_idx.size() != best_header_row.size()) {
-		auto error = CSVError::SniffingError(options.file_path);
-		error_handler->Error(error);
+	if (!options.null_padding && best_sql_types_candidates_per_column_idx.size() != best_header_row.size()) {
+		auto error =
+		    CSVError::HeaderSniffingError(options, best_header_row, best_sql_types_candidates_per_column_idx.size(),
+		                                  state_machine.dialect_options.state_machine_options.delimiter.GetValue());
+		error_handler.Error(error);
 	}
 	bool all_varchar = true;
 	bool has_header;
 
 	if (set_columns.IsSet()) {
-		has_header = DetectHeaderWithSetColumn();
+		has_header = DetectHeaderWithSetColumn(context, best_header_row, set_columns, options);
 	} else {
 		for (idx_t col = 0; col < best_header_row.size(); col++) {
 			if (!best_header_row[col].IsNull()) {
@@ -177,7 +212,7 @@ void CSVSniffer::DetectHeader() {
 			const auto &sql_type = best_sql_types_candidates_per_column_idx[col].back();
 			if (sql_type != LogicalType::VARCHAR) {
 				all_varchar = false;
-				if (!CanYouCastIt(best_header_row[col].value, sql_type, sniffer_state_machine.dialect_options,
+				if (!CanYouCastIt(context, best_header_row[col].value, sql_type, dialect_options,
 				                  best_header_row[col].IsNull(), options.decimal_separator[0])) {
 					first_row_consistent = false;
 				}
@@ -191,33 +226,31 @@ void CSVSniffer::DetectHeader() {
 		}
 	}
 
-	if (sniffer_state_machine.options.dialect_options.header.IsSetByUser()) {
+	if (options.dialect_options.header.IsSetByUser()) {
 		// Header is defined by user, use that.
-		has_header = sniffer_state_machine.options.dialect_options.header.GetValue();
+		has_header = options.dialect_options.header.GetValue();
 	}
 	// update parser info, and read, generate & set col_names based on previous findings
 	if (has_header) {
-		sniffer_state_machine.dialect_options.header = true;
-		if (sniffer_state_machine.options.null_padding &&
-		    !sniffer_state_machine.options.dialect_options.skip_rows.IsSetByUser()) {
-			if (sniffer_state_machine.dialect_options.skip_rows.GetValue() > 0) {
-				sniffer_state_machine.dialect_options.skip_rows =
-				    sniffer_state_machine.dialect_options.skip_rows.GetValue() - 1;
+		dialect_options.header = true;
+		if (options.null_padding && !options.dialect_options.skip_rows.IsSetByUser()) {
+			if (dialect_options.skip_rows.GetValue() > 0) {
+				dialect_options.skip_rows = dialect_options.skip_rows.GetValue() - 1;
 			}
 		}
 		case_insensitive_map_t<idx_t> name_collision_count;
 
 		// get header names from CSV
 		for (idx_t col = 0; col < best_header_row.size(); col++) {
-			string col_name = best_header_row[col].value.GetString();
+			string &col_name = best_header_row[col].value;
 
 			// generate name if field is empty
-			if (col_name.empty() || best_header_row[col].IsNull()) {
-				col_name = GenerateColumnName(sniffer_state_machine.dialect_options.num_cols, col);
+			if (EmptyHeader(col_name, best_header_row[col].is_null, options.normalize_names)) {
+				col_name = GenerateColumnName(dialect_options.num_cols, col);
 			}
 
 			// normalize names or at least trim whitespace
-			if (sniffer_state_machine.options.normalize_names) {
+			if (options.normalize_names) {
 				col_name = NormalizeColumnName(col_name);
 			} else {
 				col_name = TrimWhitespace(col_name);
@@ -228,27 +261,35 @@ void CSVSniffer::DetectHeader() {
 				name_collision_count[col_name] += 1;
 				col_name = col_name + "_" + to_string(name_collision_count[col_name]);
 			}
-			names.push_back(col_name);
+			detected_names.push_back(col_name);
 			name_collision_count[col_name] = 0;
 		}
-		if (best_header_row.size() < sniffer_state_machine.dialect_options.num_cols && options.null_padding) {
-			for (idx_t col = best_header_row.size(); col < sniffer_state_machine.dialect_options.num_cols; col++) {
-				names.push_back(GenerateColumnName(sniffer_state_machine.dialect_options.num_cols, col));
+		if (best_header_row.size() < dialect_options.num_cols && options.null_padding) {
+			for (idx_t col = best_header_row.size(); col < dialect_options.num_cols; col++) {
+				detected_names.push_back(GenerateColumnName(dialect_options.num_cols, col));
 			}
-		} else if (best_header_row.size() < sniffer_state_machine.dialect_options.num_cols) {
+		} else if (best_header_row.size() < dialect_options.num_cols) {
 			throw InternalException("Detected header has number of columns inferior to dialect detection");
 		}
 
 	} else {
-		sniffer_state_machine.dialect_options.header = false;
-		for (idx_t col = 0; col < sniffer_state_machine.dialect_options.num_cols; col++) {
-			names.push_back(GenerateColumnName(sniffer_state_machine.dialect_options.num_cols, col));
+		dialect_options.header = false;
+		for (idx_t col = 0; col < dialect_options.num_cols; col++) {
+			detected_names.push_back(GenerateColumnName(dialect_options.num_cols, col));
 		}
 	}
 
 	// If the user provided names, we must replace our header with the user provided names
-	for (idx_t i = 0; i < MinValue<idx_t>(names.size(), sniffer_state_machine.options.name_list.size()); i++) {
-		names[i] = sniffer_state_machine.options.name_list[i];
+	if (!options.columns_set) {
+		for (idx_t i = 0; i < MinValue<idx_t>(detected_names.size(), options.name_list.size()); i++) {
+			detected_names[i] = options.name_list[i];
+		}
 	}
+	return detected_names;
+}
+void CSVSniffer::DetectHeader() {
+	auto &sniffer_state_machine = best_candidate->GetStateMachine();
+	names = DetectHeaderInternal(buffer_manager->context, best_header_row, sniffer_state_machine, set_columns,
+	                             best_sql_types_candidates_per_column_idx, options, *error_handler);
 }
 } // namespace duckdb
