@@ -3,12 +3,14 @@
 
 namespace duckdb {
 
-void PhysicalPrepare::GetData(ExecutionContext &context, DataChunk &chunk, GlobalSourceState &gstate_p,
-                              LocalSourceState &lstate) const {
+SourceResultType PhysicalPrepare::GetData(ExecutionContext &context, DataChunk &chunk,
+                                          OperatorSourceInput &input) const {
 	auto &client = context.client;
 
 	// store the prepared statement in the context
 	ClientData::Get(client).prepared_statements[name] = prepared;
+
+	return SourceResultType::FINISHED;
 }
 
 } // namespace duckdb
