@@ -9,12 +9,6 @@ void BuiltinFunctions::RegisterOperators() {
 	Register<MultiplyFun>();
 	Register<DivideFun>();
 	Register<ModFun>();
-	Register<LeftShiftFun>();
-	Register<RightShiftFun>();
-	Register<BitwiseAndFun>();
-	Register<BitwiseOrFun>();
-	Register<BitwiseXorFun>();
-	Register<BitwiseNotFun>();
 }
 
 } // namespace duckdb
