@@ -11,15 +11,13 @@
 #include "duckdb/function/copy_function.hpp"
 #include "duckdb/function/scalar/string_functions.hpp"
 #include "duckdb/function/table/read_csv.hpp"
-#include "duckdb/parser/parsed_data/copy_info.hpp"
-#include "duckdb/parser/expression/cast_expression.hpp"
-#include "duckdb/parser/expression/function_expression.hpp"
-#include "duckdb/parser/expression/columnref_expression.hpp"
-#include "duckdb/parser/expression/constant_expression.hpp"
 #include "duckdb/parser/expression/bound_expression.hpp"
+#include "duckdb/parser/expression/cast_expression.hpp"
+#include "duckdb/parser/expression/constant_expression.hpp"
+#include "duckdb/parser/expression/function_expression.hpp"
+#include "duckdb/parser/parsed_data/copy_info.hpp"
 #include "duckdb/planner/expression/bound_reference_expression.hpp"
-#include "duckdb/execution/column_binding_resolver.hpp"
-#include "duckdb/planner/operator/logical_dummy_scan.hpp"
+
 #include <limits>
 
 namespace duckdb {
@@ -70,6 +68,14 @@ void BaseCSVData::Finalize() {
 		                options.dialect_options.state_machine_options.escape.GetValue(), "QUOTE", "ESCAPE");
 	}
 
+	// delimiter and quote must not be substrings of each other
+	AreOptionsEqual(options.dialect_options.state_machine_options.comment.GetValue(),
+	                options.dialect_options.state_machine_options.quote.GetValue(), "COMMENT", "QUOTE");
+
+	// delimiter and quote must not be substrings of each other
+	AreOptionsEqual(options.dialect_options.state_machine_options.comment.GetValue(),
+	                options.dialect_options.state_machine_options.delimiter.GetValue(), "COMMENT", "DELIMITER");
+
 	// null string and delimiter must not be substrings of each other
 	for (auto &null_str : options.null_str) {
 		if (!null_str.empty()) {
@@ -174,6 +180,21 @@ static unique_ptr<FunctionData> WriteCSVBind(ClientContext &context, CopyFunctio
 	}
 	bind_data->Finalize();
 
+	switch (bind_data->options.compression) {
+	case FileCompressionType::GZIP:
+		if (!IsFileCompressed(input.file_extension, FileCompressionType::GZIP)) {
+			input.file_extension += CompressionExtensionFromType(FileCompressionType::GZIP);
+		}
+		break;
+	case FileCompressionType::ZSTD:
+		if (!IsFileCompressed(input.file_extension, FileCompressionType::ZSTD)) {
+			input.file_extension += CompressionExtensionFromType(FileCompressionType::ZSTD);
+		}
+		break;
+	default:
+		break;
+	}
+
 	auto expressions = CreateCastExpressions(*bind_data, context, names, sql_types);
 	bind_data->cast_expressions = std::move(expressions);
 
@@ -225,14 +246,14 @@ static unique_ptr<FunctionData> ReadCSVBind(ClientContext &context, CopyInfo &in
 	options.file_path = bind_data->files[0];
 	options.name_list = expected_names;
 	options.sql_type_list = expected_types;
+	options.columns_set = true;
 	for (idx_t i = 0; i < expected_types.size(); i++) {
 		options.sql_types_per_column[expected_names[i]] = i;
 	}
 
 	if (options.auto_detect) {
 		auto buffer_manager = make_shared_ptr<CSVBufferManager>(context, options, bind_data->files[0], 0);
-		CSVSniffer sniffer(options, buffer_manager, CSVStateMachineCache::Get(context),
-		                   {&expected_types, &expected_names});
+		CSVSniffer sniffer(options, buffer_manager, CSVStateMachineCache::Get(context));
 		sniffer.SniffCSV();
 	}
 	bind_data->FinalizeRead(context);
@@ -432,11 +453,6 @@ static unique_ptr<GlobalFunctionData> WriteCSVInitializeGlobal(ClientContext &co
 	return std::move(global_data);
 }
 
-idx_t WriteCSVFileSize(GlobalFunctionData &gstate) {
-	auto &global_state = gstate.Cast<GlobalWriteCSVData>();
-	return global_state.FileSize();
-}
-
 static void WriteCSVChunkInternal(ClientContext &context, FunctionData &bind_data, DataChunk &cast_chunk,
                                   MemoryStream &writer, DataChunk &input, bool &written_anything,
                                   ExpressionExecutor &executor) {
@@ -592,6 +608,18 @@ void WriteCSVFlushBatch(ClientContext &context, FunctionData &bind_data, GlobalF
 	writer.Rewind();
 }
 
+//===--------------------------------------------------------------------===//
+// File rotation
+//===--------------------------------------------------------------------===//
+bool WriteCSVRotateFiles(FunctionData &, const optional_idx &file_size_bytes) {
+	return file_size_bytes.IsValid();
+}
+
+bool WriteCSVRotateNextFile(GlobalFunctionData &gstate, FunctionData &, const optional_idx &file_size_bytes) {
+	auto &global_state = gstate.Cast<GlobalWriteCSVData>();
+	return global_state.FileSize() > file_size_bytes.GetIndex();
+}
+
 void CSVCopyFunction::RegisterFunction(BuiltinFunctions &set) {
 	CopyFunction info("csv");
 	info.copy_to_bind = WriteCSVBind;
@@ -603,7 +631,8 @@ void CSVCopyFunction::RegisterFunction(BuiltinFunctions &set) {
 	info.execution_mode = WriteCSVExecutionMode;
 	info.prepare_batch = WriteCSVPrepareBatch;
 	info.flush_batch = WriteCSVFlushBatch;
-	info.file_size_bytes = WriteCSVFileSize;
+	info.rotate_files = WriteCSVRotateFiles;
+	info.rotate_next_file = WriteCSVRotateNextFile;
 
 	info.copy_from_bind = ReadCSVBind;
 	info.copy_from_function = ReadCSVTableFunction::GetFunction();
