@@ -36,6 +36,10 @@ static unique_ptr<GlobalTableFunctionState> CSVSniffInitGlobal(ClientContext &co
 static unique_ptr<FunctionData> CSVSniffBind(ClientContext &context, TableFunctionBindInput &input,
                                              vector<LogicalType> &return_types, vector<string> &names) {
 	auto result = make_uniq<CSVSniffFunctionData>();
+	auto &config = DBConfig::GetConfig(context);
+	if (!config.options.enable_external_access) {
+		throw PermissionException("sniff_csv is disabled through configuration");
+	}
 	result->path = input.inputs[0].ToString();
 	auto it = input.named_parameters.find("auto_detect");
 	if (it != input.named_parameters.end()) {
@@ -45,7 +49,7 @@ static unique_ptr<FunctionData> CSVSniffBind(ClientContext &context, TableFuncti
 		// otherwise remove it
 		input.named_parameters.erase("auto_detect");
 	}
-	result->options.FromNamedParameters(input.named_parameters, context, result->return_types_csv, result->names_csv);
+	result->options.FromNamedParameters(input.named_parameters, context);
 	// We want to return the whole CSV Configuration
 	// 1. Delimiter
 	return_types.emplace_back(LogicalType::VARCHAR);
@@ -59,25 +63,30 @@ static unique_ptr<FunctionData> CSVSniffBind(ClientContext &context, TableFuncti
 	// 4. NewLine Delimiter
 	return_types.emplace_back(LogicalType::VARCHAR);
 	names.emplace_back("NewLineDelimiter");
-	// 5. Skip Rows
+	// 5. Comment
+	return_types.emplace_back(LogicalType::VARCHAR);
+	names.emplace_back("Comment");
+	// 6. Skip Rows
 	return_types.emplace_back(LogicalType::UINTEGER);
 	names.emplace_back("SkipRows");
-	// 6. Has Header
+	// 7. Has Header
 	return_types.emplace_back(LogicalType::BOOLEAN);
 	names.emplace_back("HasHeader");
-	// 7. List<Struct<Column-Name:Types>>
-	return_types.emplace_back(LogicalType::VARCHAR);
+	// 8. List<Struct<Column-Name:Types>>
+	child_list_t<LogicalType> struct_children {{"name", LogicalType::VARCHAR}, {"type", LogicalType::VARCHAR}};
+	auto list_child = LogicalType::STRUCT(struct_children);
+	return_types.emplace_back(LogicalType::LIST(list_child));
 	names.emplace_back("Columns");
-	// 8. Date Format
+	// 9. Date Format
 	return_types.emplace_back(LogicalType::VARCHAR);
 	names.emplace_back("DateFormat");
-	// 9. Timestamp Format
+	// 10. Timestamp Format
 	return_types.emplace_back(LogicalType::VARCHAR);
 	names.emplace_back("TimestampFormat");
-	// 10. CSV read function with all the options used
+	// 11. CSV read function with all the options used
 	return_types.emplace_back(LogicalType::VARCHAR);
 	names.emplace_back("UserArguments");
-	// 11. CSV read function with all the options used
+	// 12. CSV read function with all the options used
 	return_types.emplace_back(LogicalType::VARCHAR);
 	names.emplace_back("Prompt");
 	return std::move(result);
@@ -101,18 +110,20 @@ static void CSVSniffFunction(ClientContext &context, TableFunctionInput &data_p,
 	const CSVSniffFunctionData &data = data_p.bind_data->Cast<CSVSniffFunctionData>();
 	auto &fs = duckdb::FileSystem::GetFileSystem(context);
 
-	if (data.path.rfind("http://", 0) != 0 && data.path.rfind("https://", 0) != 0 && fs.HasGlob(data.path)) {
-		throw NotImplementedException("sniff_csv does not operate on globs yet");
+	auto paths = fs.GlobFiles(data.path, context, FileGlobOptions::DISALLOW_EMPTY);
+	if (paths.size() > 1) {
+		throw NotImplementedException("sniff_csv does not operate on more than one file yet");
 	}
 
 	// We must run the sniffer.
 	auto sniffer_options = data.options;
-	sniffer_options.file_path = data.path;
+	sniffer_options.file_path = paths[0];
 
-	auto buffer_manager = make_shared<CSVBufferManager>(context, sniffer_options, sniffer_options.file_path, 0);
+	auto buffer_manager = make_shared_ptr<CSVBufferManager>(context, sniffer_options, sniffer_options.file_path, 0);
 	if (sniffer_options.name_list.empty()) {
 		sniffer_options.name_list = data.names_csv;
 	}
+
 	if (sniffer_options.sql_type_list.empty()) {
 		sniffer_options.sql_type_list = data.return_types_csv;
 	}
@@ -135,60 +146,67 @@ static void CSVSniffFunction(ClientContext &context, TableFunctionInput &data_p,
 	// 4. NewLine Delimiter
 	auto new_line_identifier = sniffer_options.NewLineIdentifierToString();
 	output.SetValue(3, 0, new_line_identifier);
-	// 5. Skip Rows
-	output.SetValue(4, 0, Value::UINTEGER(NumericCast<uint32_t>(sniffer_options.dialect_options.skip_rows.GetValue())));
-	// 6. Has Header
+	// 5. Comment
+	str_opt = sniffer_options.dialect_options.state_machine_options.comment.GetValue();
+	output.SetValue(4, 0, str_opt);
+	// 6. Skip Rows
+	output.SetValue(5, 0, Value::UINTEGER(NumericCast<uint32_t>(sniffer_options.dialect_options.skip_rows.GetValue())));
+	// 7. Has Header
 	auto has_header = Value::BOOLEAN(sniffer_options.dialect_options.header.GetValue()).ToString();
-	output.SetValue(5, 0, has_header);
-	// 7. List<Struct<Column-Name:Types>> {'col1': 'INTEGER', 'col2': 'VARCHAR'}
+	output.SetValue(6, 0, has_header);
+	// 8. List<Struct<Column-Name:Types>> {'col1': 'INTEGER', 'col2': 'VARCHAR'}
+	vector<Value> values;
 	std::ostringstream columns;
 	columns << "{";
 	for (idx_t i = 0; i < sniffer_result.return_types.size(); i++) {
+		child_list_t<Value> struct_children {{"name", sniffer_result.names[i]},
+		                                     {"type", {sniffer_result.return_types[i].ToString()}}};
+		values.emplace_back(Value::STRUCT(struct_children));
 		columns << "'" << sniffer_result.names[i] << "': '" << sniffer_result.return_types[i].ToString() << "'";
 		if (i != sniffer_result.return_types.size() - 1) {
 			columns << separator;
 		}
 	}
 	columns << "}";
-	output.SetValue(6, 0, columns.str());
-	// 8. Date Format
+	output.SetValue(7, 0, Value::LIST(values));
+	// 9. Date Format
 	auto date_format = sniffer_options.dialect_options.date_format[LogicalType::DATE].GetValue();
 	if (!date_format.Empty()) {
-		output.SetValue(7, 0, date_format.format_specifier);
+		output.SetValue(8, 0, date_format.format_specifier);
 	} else {
 		bool has_date = false;
 		for (auto &c_type : sniffer_result.return_types) {
 			// Must be ISO 8601
 			if (c_type.id() == LogicalTypeId::DATE) {
-				output.SetValue(7, 0, Value("%Y-%m-%d"));
+				output.SetValue(8, 0, Value("%Y-%m-%d"));
 				has_date = true;
 			}
 		}
 		if (!has_date) {
-			output.SetValue(7, 0, Value(nullptr));
+			output.SetValue(8, 0, Value(nullptr));
 		}
 	}
 
-	// 9. Timestamp Format
+	// 10. Timestamp Format
 	auto timestamp_format = sniffer_options.dialect_options.date_format[LogicalType::TIMESTAMP].GetValue();
 	if (!timestamp_format.Empty()) {
-		output.SetValue(8, 0, timestamp_format.format_specifier);
+		output.SetValue(9, 0, timestamp_format.format_specifier);
 	} else {
-		output.SetValue(8, 0, Value(nullptr));
+		output.SetValue(9, 0, Value(nullptr));
 	}
 
-	// 10. The Extra User Arguments
+	// 11. The Extra User Arguments
 	if (data.options.user_defined_parameters.empty()) {
-		output.SetValue(9, 0, Value());
+		output.SetValue(10, 0, Value());
 	} else {
-		output.SetValue(9, 0, Value(data.options.user_defined_parameters));
+		output.SetValue(10, 0, Value(data.options.user_defined_parameters));
 	}
 
-	// 11. csv_read string
+	// 12. csv_read string
 	std::ostringstream csv_read;
 
 	// Base, Path and auto_detect=false
-	csv_read << "FROM read_csv('" << data.path << "'" << separator << "auto_detect=false" << separator;
+	csv_read << "FROM read_csv('" << paths[0] << "'" << separator << "auto_detect=false" << separator;
 	// 10.1. Delimiter
 	if (!sniffer_options.dialect_options.state_machine_options.delimiter.IsSetByUser()) {
 		csv_read << "delim="
@@ -218,13 +236,21 @@ static void CSVSniffFunction(ClientContext &context, TableFunctionInput &data_p,
 	if (!sniffer_options.dialect_options.skip_rows.IsSetByUser()) {
 		csv_read << "skip=" << sniffer_options.dialect_options.skip_rows.GetValue() << separator;
 	}
-	// 11.6. Has Header
+
+	// 11.6. Comment
+	if (!sniffer_options.dialect_options.state_machine_options.comment.IsSetByUser()) {
+		csv_read << "comment="
+		         << "'" << FormatOptions(sniffer_options.dialect_options.state_machine_options.comment.GetValue())
+		         << "'" << separator;
+	}
+
+	// 11.7. Has Header
 	if (!sniffer_options.dialect_options.header.IsSetByUser()) {
 		csv_read << "header=" << has_header << separator;
 	}
-	// 11.7. column={'col1': 'INTEGER', 'col2': 'VARCHAR'}
+	// 11.8. column={'col1': 'INTEGER', 'col2': 'VARCHAR'}
 	csv_read << "columns=" << columns.str();
-	// 11.8. Date Format
+	// 11.9. Date Format
 	if (!sniffer_options.dialect_options.date_format[LogicalType::DATE].IsSetByUser()) {
 		if (!sniffer_options.dialect_options.date_format[LogicalType::DATE].GetValue().format_specifier.empty()) {
 			csv_read << separator << "dateformat="
@@ -242,7 +268,7 @@ static void CSVSniffFunction(ClientContext &context, TableFunctionInput &data_p,
 			}
 		}
 	}
-	// 11.9. Timestamp Format
+	// 11.10. Timestamp Format
 	if (!sniffer_options.dialect_options.date_format[LogicalType::TIMESTAMP].IsSetByUser()) {
 		if (!sniffer_options.dialect_options.date_format[LogicalType::TIMESTAMP].GetValue().format_specifier.empty()) {
 			csv_read << separator << "timestampformat="
@@ -251,12 +277,12 @@ static void CSVSniffFunction(ClientContext &context, TableFunctionInput &data_p,
 			         << "'";
 		}
 	}
-	// 11.10 User Arguments
+	// 11.11 User Arguments
 	if (!data.options.user_defined_parameters.empty()) {
 		csv_read << separator << data.options.user_defined_parameters;
 	}
 	csv_read << ");";
-	output.SetValue(10, 0, csv_read.str());
+	output.SetValue(11, 0, csv_read.str());
 	global_state.done = true;
 }
 
