@@ -26,6 +26,7 @@ class HTTPState;
 class QueryProfiler;
 class PreparedStatementData;
 class SchemaCatalogEntry;
+class HTTPLogger;
 struct RandomEngine;
 
 struct ClientData {
@@ -35,6 +36,9 @@ struct ClientData {
 	//! Query profiler
 	shared_ptr<QueryProfiler> profiler;
 
+	//! HTTP logger
+	shared_ptr<HTTPLogger> http_logger;
+
 	//! The set of temporary objects that belong to this client
 	shared_ptr<AttachedDatabase> temporary_objects;
 	//! The set of bound prepared statements that belong to this client
