@@ -26,6 +26,12 @@ public:
 	}
 };
 
+enum class ExtensionABIType : uint8_t {
+	UNKNOWN = 0,
+	CPP = 1,
+	C_STRUCT = 2,
+};
+
 //! The parsed extension metadata footer
 struct ParsedExtensionMetaData {
 	static constexpr const idx_t FOOTER_SIZE = 512;
@@ -35,8 +41,13 @@ struct ParsedExtensionMetaData {
 
 	string magic_value;
 
+	ExtensionABIType abi_type;
+
 	string platform;
+	// (only for ExtensionABIType::CPP) the DuckDB version this extension is compiled for
 	string duckdb_version;
+	// (only for ExtensionABIType::C_STRUCT) the CAPI version of the C_STRUCT
+	string duckdb_capi_version;
 	string extension_version;
 	string signature;
 
@@ -48,4 +59,13 @@ struct ParsedExtensionMetaData {
 	string GetInvalidMetadataError();
 };
 
+struct VersioningUtils {
+	//! Note: only supports format v{major}.{minor}.{patch}
+	static bool ParseSemver(string &semver, idx_t &major_out, idx_t &minor_out, idx_t &patch_out);
+
+	//! Note: only supports format v{major}.{minor}.{patch}
+	static bool IsSupportedCAPIVersion(string &capi_version_string);
+	static bool IsSupportedCAPIVersion(idx_t major, idx_t minor, idx_t patch);
+};
+
 } // namespace duckdb
