@@ -31,11 +31,16 @@ public:
 	unique_ptr<ParsedExpression> expr;
 	//! Whether or not this is a COLUMNS expression
 	bool columns = false;
+	//! Whether the columns are unpacked
+	bool unpacked = false;
 
 public:
 	string ToString() const override;
 
 	static bool Equal(const StarExpression &a, const StarExpression &b);
+	static bool IsStar(const ParsedExpression &a);
+	static bool IsColumns(const ParsedExpression &a);
+	static bool IsColumnsUnpacked(const ParsedExpression &a);
 
 	unique_ptr<ParsedExpression> Copy() const override;
 
