@@ -10,13 +10,15 @@
 
 #include "duckdb/common/atomic.hpp"
 #include "duckdb/common/common.hpp"
+#include "duckdb/common/enums/destroy_buffer_upon.hpp"
+#include "duckdb/common/enums/memory_tag.hpp"
+#include "duckdb/common/file_buffer.hpp"
 #include "duckdb/common/mutex.hpp"
 #include "duckdb/common/numeric_utils.hpp"
 #include "duckdb/storage/storage_info.hpp"
-#include "duckdb/common/file_buffer.hpp"
-#include "duckdb/common/enums/memory_tag.hpp"
 
 namespace duckdb {
+
 class BlockManager;
 class BufferHandle;
 class BufferPool;
@@ -59,11 +61,12 @@ class BlockHandle {
 	friend class BufferManager;
 	friend class StandardBufferManager;
 	friend class BufferPool;
+	friend struct EvictionQueue;
 
 public:
 	BlockHandle(BlockManager &block_manager, block_id_t block_id, MemoryTag tag);
 	BlockHandle(BlockManager &block_manager, block_id_t block_id, MemoryTag tag, unique_ptr<FileBuffer> buffer,
-	            bool can_destroy, idx_t block_size, BufferPoolReservation &&reservation);
+	            DestroyBufferUpon destroy_buffer_upon, idx_t block_size, BufferPoolReservation &&reservation);
 	~BlockHandle();
 
 	BlockManager &block_manager;
@@ -93,8 +96,17 @@ public:
 		unswizzled = unswizzler;
 	}
 
-	inline void SetCanDestroy(bool can_destroy_p) {
-		can_destroy = can_destroy_p;
+	inline void SetDestroyBufferUpon(DestroyBufferUpon destroy_buffer_upon_p) {
+		lock_guard<mutex> guard(lock);
+		destroy_buffer_upon = destroy_buffer_upon_p;
+	}
+
+	inline bool MustAddToEvictionQueue() const {
+		return destroy_buffer_upon != DestroyBufferUpon::UNPIN;
+	}
+
+	inline bool MustWriteToTemporaryFile() const {
+		return destroy_buffer_upon == DestroyBufferUpon::BLOCK;
 	}
 
 	inline const idx_t &GetMemoryUsage() const {
@@ -106,6 +118,8 @@ public:
 
 private:
 	static BufferHandle Load(shared_ptr<BlockHandle> &handle, unique_ptr<FileBuffer> buffer = nullptr);
+	static BufferHandle LoadFromBuffer(shared_ptr<BlockHandle> &handle, data_ptr_t data,
+	                                   unique_ptr<FileBuffer> reusable_buffer);
 	unique_ptr<FileBuffer> UnloadAndTakeBlock();
 	void Unload();
 	bool CanUnload();
@@ -126,8 +140,8 @@ private:
 	atomic<idx_t> eviction_seq_num;
 	//! LRU timestamp (for age-based eviction)
 	atomic<int64_t> lru_timestamp_msec;
-	//! Whether or not the buffer can be destroyed (only used for temporary buffers)
-	bool can_destroy;
+	//! When to destroy the data buffer
+	DestroyBufferUpon destroy_buffer_upon;
 	//! The memory usage of the block (when loaded). If we are pinning/loading
 	//! an unloaded block, this tells us how much memory to reserve.
 	idx_t memory_usage;
