@@ -25,17 +25,13 @@ class Transaction;
 //! The MetaTransaction manages multiple transactions for different attached databases
 class MetaTransaction {
 public:
-	DUCKDB_API MetaTransaction(ClientContext &context, timestamp_t start_timestamp, idx_t catalog_version);
+	DUCKDB_API MetaTransaction(ClientContext &context, timestamp_t start_timestamp);
 
 	ClientContext &context;
 	//! The timestamp when the transaction started
 	timestamp_t start_timestamp;
-	//! The catalog version when the transaction was started
-	idx_t catalog_version;
 	//! The validity checker of the transaction
 	ValidChecker transaction_validity;
-	//! Whether or not any transaction have made modifications
-	bool read_only;
 	//! The active query number
 	transaction_t active_query;
 
@@ -55,6 +51,8 @@ public:
 	idx_t GetActiveQuery();
 	void SetActiveQuery(transaction_t query_number);
 
+	void SetReadOnly();
+	bool IsReadOnly() const;
 	void ModifyDatabase(AttachedDatabase &db);
 	optional_ptr<AttachedDatabase> ModifiedDatabase() {
 		return modified_database;
@@ -72,6 +70,8 @@ private:
 	vector<reference<AttachedDatabase>> all_transactions;
 	//! The database we are modifying - we can only modify one database per transaction
 	optional_ptr<AttachedDatabase> modified_database;
+	//! Whether or not the meta transaction is marked as read only
+	bool is_read_only;
 };
 
 } // namespace duckdb
