@@ -1,7 +1,9 @@
 #include "duckdb/main/config.hpp"
 
-#include "duckdb/common/operator/multiply.hpp"
+#include "duckdb/common/cgroups.hpp"
+#include "duckdb/common/file_system.hpp"
 #include "duckdb/common/operator/cast_operators.hpp"
+#include "duckdb/common/operator/multiply.hpp"
 #include "duckdb/common/string_util.hpp"
 #include "duckdb/main/settings.hpp"
 #include "duckdb/storage/storage_extension.hpp"
@@ -57,8 +59,10 @@ bool DBConfigOptions::debug_print_bindings = false;
 static const ConfigurationOption internal_options[] = {
     DUCKDB_GLOBAL(AccessModeSetting),
     DUCKDB_GLOBAL(AllowPersistentSecrets),
+    DUCKDB_GLOBAL(CatalogErrorMaxSchema),
     DUCKDB_GLOBAL(CheckpointThresholdSetting),
     DUCKDB_GLOBAL(DebugCheckpointAbort),
+    DUCKDB_GLOBAL(DebugSkipCheckpointOnCommit),
     DUCKDB_GLOBAL(StorageCompatibilityVersion),
     DUCKDB_LOCAL(DebugForceExternal),
     DUCKDB_LOCAL(DebugForceNoCrossProduct),
@@ -93,18 +97,27 @@ static const ConfigurationOption internal_options[] = {
     DUCKDB_GLOBAL(ForceCompressionSetting),
     DUCKDB_GLOBAL(ForceBitpackingModeSetting),
     DUCKDB_LOCAL(HomeDirectorySetting),
+    DUCKDB_GLOBAL(HTTPProxy),
+    DUCKDB_GLOBAL(HTTPProxyUsername),
+    DUCKDB_GLOBAL(HTTPProxyPassword),
     DUCKDB_LOCAL(LogQueryPathSetting),
     DUCKDB_GLOBAL(EnableMacrosDependencies),
     DUCKDB_GLOBAL(EnableViewDependencies),
     DUCKDB_GLOBAL(LockConfigurationSetting),
+    DUCKDB_LOCAL(IEEEFloatingPointOpsSetting),
     DUCKDB_GLOBAL(ImmediateTransactionModeSetting),
     DUCKDB_LOCAL(IntegerDivisionSetting),
     DUCKDB_LOCAL(MaximumExpressionDepthSetting),
+    DUCKDB_LOCAL(StreamingBufferSize),
     DUCKDB_GLOBAL(MaximumMemorySetting),
     DUCKDB_GLOBAL(MaximumTempDirectorySize),
+    DUCKDB_GLOBAL(MaximumVacuumTasks),
+    DUCKDB_LOCAL(MergeJoinThreshold),
+    DUCKDB_LOCAL(NestedLoopJoinThreshold),
     DUCKDB_GLOBAL(OldImplicitCasting),
     DUCKDB_GLOBAL_ALIAS("memory_limit", MaximumMemorySetting),
     DUCKDB_GLOBAL_ALIAS("null_order", DefaultNullOrderSetting),
+    DUCKDB_LOCAL(OrderByNonIntegerLiteral),
     DUCKDB_LOCAL(OrderedAggregateThreshold),
     DUCKDB_GLOBAL(PasswordSetting),
     DUCKDB_LOCAL(PerfectHashThresholdSetting),
@@ -115,22 +128,33 @@ static const ConfigurationOption internal_options[] = {
     DUCKDB_LOCAL(ProfileOutputSetting),
     DUCKDB_LOCAL(ProfilingModeSetting),
     DUCKDB_LOCAL_ALIAS("profiling_output", ProfileOutputSetting),
+    DUCKDB_LOCAL(CustomProfilingSettings),
     DUCKDB_LOCAL(ProgressBarTimeSetting),
     DUCKDB_LOCAL(SchemaSetting),
     DUCKDB_LOCAL(SearchPathSetting),
+    DUCKDB_LOCAL(ScalarSubqueryErrorOnMultipleRows),
     DUCKDB_GLOBAL(SecretDirectorySetting),
     DUCKDB_GLOBAL(DefaultSecretStorage),
     DUCKDB_GLOBAL(TempDirectorySetting),
     DUCKDB_GLOBAL(ThreadsSetting),
     DUCKDB_GLOBAL(UsernameSetting),
     DUCKDB_GLOBAL(ExportLargeBufferArrow),
+    DUCKDB_GLOBAL(ArrowOutputListView),
+    DUCKDB_GLOBAL(LosslessConversionArrow),
+    DUCKDB_GLOBAL(ProduceArrowStringView),
     DUCKDB_GLOBAL_ALIAS("user", UsernameSetting),
     DUCKDB_GLOBAL_ALIAS("wal_autocheckpoint", CheckpointThresholdSetting),
     DUCKDB_GLOBAL_ALIAS("worker_threads", ThreadsSetting),
-    DUCKDB_GLOBAL(FlushAllocatorSetting),
+    DUCKDB_GLOBAL(AllocatorFlushThreshold),
+    DUCKDB_GLOBAL(AllocatorBulkDeallocationFlushThreshold),
+    DUCKDB_GLOBAL(AllocatorBackgroundThreadsSetting),
     DUCKDB_GLOBAL(DuckDBApiSetting),
     DUCKDB_GLOBAL(CustomUserAgentSetting),
     DUCKDB_LOCAL(PartitionedWriteFlushThreshold),
+    DUCKDB_LOCAL(PartitionedWriteMaxOpenFiles),
+    DUCKDB_GLOBAL(DefaultBlockAllocSize),
+    DUCKDB_GLOBAL(IndexScanPercentage),
+    DUCKDB_GLOBAL(IndexScanMaxCount),
     DUCKDB_LOCAL(EnableHTTPLoggingSetting),
     DUCKDB_LOCAL(HTTPLoggingOutputSetting),
     FINAL_SETTING};
@@ -184,6 +208,10 @@ void DBConfig::SetOption(const ConfigurationOption &option, const Value &value)
 }
 
 void DBConfig::SetOptionByName(const string &name, const Value &value) {
+	if (is_user_config) {
+		// for user config we just set the option in the `user_options`
+		options.user_options[name] = value;
+	}
 	auto option = DBConfig::GetOptionByName(name);
 	if (option) {
 		SetOption(*option, value);
@@ -274,14 +302,22 @@ CastFunctionSet &DBConfig::GetCastFunctions() {
 	return *cast_functions;
 }
 
+CollationBinding &DBConfig::GetCollationBinding() {
+	return *collation_bindings;
+}
+
 IndexTypeSet &DBConfig::GetIndexTypes() {
 	return *index_types;
 }
 
 void DBConfig::SetDefaultMaxMemory() {
-	auto memory = FileSystem::GetAvailableMemory();
-	if (memory.IsValid()) {
-		options.maximum_memory = memory.GetIndex() * 8 / 10;
+	auto memory = GetSystemAvailableMemory(*file_system);
+	if (memory == DBConfigOptions().maximum_memory) {
+		// If GetSystemAvailableMemory returned the default, use it as is
+		options.maximum_memory = memory;
+	} else {
+		// Otherwise, use 80% of the available memory
+		options.maximum_memory = memory * 8 / 10;
 	}
 }
 
@@ -307,69 +343,52 @@ void DBConfig::CheckLock(const string &name) {
 	throw InvalidInputException("Cannot change configuration option \"%s\" - the configuration has been locked", name);
 }
 
-idx_t CGroupBandwidthQuota(idx_t physical_cores, FileSystem &fs) {
-	static constexpr const char *CPU_MAX = "/sys/fs/cgroup/cpu.max";
-	static constexpr const char *CFS_QUOTA = "/sys/fs/cgroup/cpu/cpu.cfs_quota_us";
-	static constexpr const char *CFS_PERIOD = "/sys/fs/cgroup/cpu/cpu.cfs_period_us";
-
-	int64_t quota, period;
-	char byte_buffer[1000];
-	unique_ptr<FileHandle> handle;
-	int64_t read_bytes;
-
-	if (fs.FileExists(CPU_MAX)) {
-		// cgroup v2
-		// https://www.kernel.org/doc/html/latest/admin-guide/cgroup-v2.html
-		handle = fs.OpenFile(CPU_MAX, FileFlags::FILE_FLAGS_READ);
-		read_bytes = fs.Read(*handle, (void *)byte_buffer, 999);
-		byte_buffer[read_bytes] = '\0';
-		if (std::sscanf(byte_buffer, "%" SCNd64 " %" SCNd64 "", &quota, &period) != 2) {
-			return physical_cores;
-		}
-	} else if (fs.FileExists(CFS_QUOTA) && fs.FileExists(CFS_PERIOD)) {
-		// cgroup v1
-		// https://www.kernel.org/doc/html/latest/scheduler/sched-bwc.html#management
-
-		// Read the quota, this indicates how many microseconds the CPU can be utilized by this cgroup per period
-		handle = fs.OpenFile(CFS_QUOTA, FileFlags::FILE_FLAGS_READ);
-		read_bytes = fs.Read(*handle, (void *)byte_buffer, 999);
-		byte_buffer[read_bytes] = '\0';
-		if (std::sscanf(byte_buffer, "%" SCNd64 "", &quota) != 1) {
-			return physical_cores;
-		}
-
-		// Read the time period, a cgroup can utilize the CPU up to quota microseconds every period
-		handle = fs.OpenFile(CFS_PERIOD, FileFlags::FILE_FLAGS_READ);
-		read_bytes = fs.Read(*handle, (void *)byte_buffer, 999);
-		byte_buffer[read_bytes] = '\0';
-		if (std::sscanf(byte_buffer, "%" SCNd64 "", &period) != 1) {
-			return physical_cores;
-		}
-	} else {
-		// No cgroup quota
-		return physical_cores;
-	}
-	if (quota > 0 && period > 0) {
-		return idx_t(std::ceil((double)quota / (double)period));
-	} else {
-		return physical_cores;
-	}
-}
-
 idx_t DBConfig::GetSystemMaxThreads(FileSystem &fs) {
-#ifndef DUCKDB_NO_THREADS
+#ifdef DUCKDB_NO_THREADS
+	return 1;
+#else
 	idx_t physical_cores = std::thread::hardware_concurrency();
 #ifdef __linux__
-	auto cores_available_per_period = CGroupBandwidthQuota(physical_cores, fs);
-	return MaxValue<idx_t>(cores_available_per_period, 1);
+	if (const char *slurm_cpus = getenv("SLURM_CPUS_ON_NODE")) {
+		idx_t slurm_threads;
+		if (TryCast::Operation<string_t, idx_t>(string_t(slurm_cpus), slurm_threads)) {
+			return MaxValue<idx_t>(slurm_threads, 1);
+		}
+	}
+	return MaxValue<idx_t>(CGroups::GetCPULimit(fs, physical_cores), 1);
 #else
 	return physical_cores;
 #endif
-#else
-	return 1;
 #endif
 }
 
+idx_t DBConfig::GetSystemAvailableMemory(FileSystem &fs) {
+	// Check SLURM environment variables first
+	const char *slurm_mem_per_node = getenv("SLURM_MEM_PER_NODE");
+	const char *slurm_mem_per_cpu = getenv("SLURM_MEM_PER_CPU");
+
+	if (slurm_mem_per_node) {
+		return ParseMemoryLimitSlurm(slurm_mem_per_node);
+	} else if (slurm_mem_per_cpu) {
+		idx_t mem_per_cpu = ParseMemoryLimitSlurm(slurm_mem_per_cpu);
+		idx_t num_threads = GetSystemMaxThreads(fs);
+		return mem_per_cpu * num_threads;
+	}
+
+	// Check cgroup memory limit
+	auto cgroup_memory_limit = CGroups::GetMemoryLimit(fs);
+	if (cgroup_memory_limit.IsValid()) {
+		return cgroup_memory_limit.GetIndex();
+	}
+
+	// Fall back to system memory detection
+	auto memory = FileSystem::GetAvailableMemory();
+	if (!memory.IsValid()) {
+		return DBConfigOptions().maximum_memory;
+	}
+	return memory.GetIndex();
+}
+
 idx_t DBConfig::ParseMemoryLimit(const string &arg) {
 	if (arg[0] == '-' || arg == "null" || arg == "none") {
 		// infinite
@@ -429,12 +448,45 @@ idx_t DBConfig::ParseMemoryLimit(const string &arg) {
 		throw ParserException("Unknown unit for memory_limit: %s (expected: KB, MB, GB, TB for 1000^i units or KiB, "
 		                      "MiB, GiB, TiB for 1024^i unites)");
 	}
-	return NumericCast<idx_t>(multiplier * limit);
+	return LossyNumericCast<idx_t>(static_cast<double>(multiplier) * limit);
+}
+
+idx_t DBConfig::ParseMemoryLimitSlurm(const string &arg) {
+	if (arg.empty()) {
+		return 0;
+	}
+
+	string number_str = arg;
+	idx_t multiplier = 1000LL * 1000LL; // Default to MB if no unit specified
+
+	// Check for SLURM-style suffixes
+	if (arg.back() == 'K' || arg.back() == 'k') {
+		number_str = arg.substr(0, arg.size() - 1);
+		multiplier = 1000LL;
+	} else if (arg.back() == 'M' || arg.back() == 'm') {
+		number_str = arg.substr(0, arg.size() - 1);
+		multiplier = 1000LL * 1000LL;
+	} else if (arg.back() == 'G' || arg.back() == 'g') {
+		number_str = arg.substr(0, arg.size() - 1);
+		multiplier = 1000LL * 1000LL * 1000LL;
+	} else if (arg.back() == 'T' || arg.back() == 't') {
+		number_str = arg.substr(0, arg.size() - 1);
+		multiplier = 1000LL * 1000LL * 1000LL * 1000LL;
+	}
+
+	// Parse the number
+	double limit = Cast::Operation<string_t, double>(string_t(number_str));
+
+	if (limit < 0) {
+		return NumericLimits<idx_t>::Maximum();
+	}
+
+	return LossyNumericCast<idx_t>(static_cast<double>(multiplier) * limit);
 }
 
 // Right now we only really care about access mode when comparing DBConfigs
 bool DBConfigOptions::operator==(const DBConfigOptions &other) const {
-	return other.access_mode == access_mode;
+	return other.access_mode == access_mode && other.user_options == user_options;
 }
 
 bool DBConfig::operator==(const DBConfig &other) {
@@ -470,7 +522,7 @@ OrderByNullType DBConfig::ResolveNullOrder(OrderType order_type, OrderByNullType
 	}
 }
 
-const std::string DBConfig::UserAgent() const {
+const string DBConfig::UserAgent() const {
 	auto user_agent = GetDefaultUserAgent();
 
 	if (!options.duckdb_api.empty()) {
@@ -506,11 +558,17 @@ SerializationCompatibility SerializationCompatibility::Default() {
 	auto res = FromString("latest");
 	res.manually_set = false;
 	return res;
+#else
+#ifdef DUCKDB_LATEST_STORAGE
+	auto res = FromString("latest");
+	res.manually_set = false;
+	return res;
 #else
 	auto res = FromString("v0.10.2");
 	res.manually_set = false;
 	return res;
 #endif
+#endif
 }
 
 SerializationCompatibility SerializationCompatibility::Latest() {
