@@ -14,6 +14,7 @@
 #include "duckdb/main/client_context.hpp"
 #include "duckdb/main/extension_helper.hpp"
 #include "duckdb/main/secret/secret_storage.hpp"
+#include "duckdb/main/secret/default_secrets.hpp"
 #include "duckdb/parser/parsed_data/create_secret_info.hpp"
 #include "duckdb/parser/statement/create_statement.hpp"
 #include "duckdb/planner/operator/logical_create_secret.hpp"
@@ -55,6 +56,16 @@ void SecretManager::Initialize(DatabaseInstance &db) {
 
 	// Store the current db for enabling autoloading
 	this->db = &db;
+
+	// Register default types
+	for (auto &type : CreateHTTPSecretFunctions::GetDefaultSecretTypes()) {
+		RegisterSecretTypeInternal(type);
+	}
+
+	// Register default functions
+	for (auto &function : CreateHTTPSecretFunctions::GetDefaultSecretFunctions()) {
+		RegisterSecretFunctionInternal(function, OnCreateConflict::ERROR_ON_CONFLICT);
+	}
 }
 
 void SecretManager::LoadSecretStorage(unique_ptr<SecretStorage> storage) {
@@ -79,7 +90,7 @@ void SecretManager::LoadSecretStorageInternal(unique_ptr<SecretStorage> storage)
 }
 
 // FIXME: use serialization scripts?
-unique_ptr<BaseSecret> SecretManager::DeserializeSecret(Deserializer &deserializer) {
+unique_ptr<BaseSecret> SecretManager::DeserializeSecret(Deserializer &deserializer, const string &secret_path) {
 	auto type = deserializer.ReadProperty<string>(100, "type");
 	auto provider = deserializer.ReadProperty<string>(101, "provider");
 	auto name = deserializer.ReadProperty<string>(102, "name");
@@ -87,35 +98,27 @@ unique_ptr<BaseSecret> SecretManager::DeserializeSecret(Deserializer &deserializ
 	deserializer.ReadList(103, "scope",
 	                      [&](Deserializer::List &list, idx_t i) { scope.push_back(list.ReadElement<string>()); });
 
-	auto secret_type = LookupTypeInternal(type);
+	SecretType deserialized_type;
+	if (!TryLookupTypeInternal(type, deserialized_type)) {
+		ThrowTypeNotFoundError(type, secret_path);
+	}
 
-	if (!secret_type.deserializer) {
+	if (!deserialized_type.deserializer) {
 		throw InternalException(
 		    "Attempted to deserialize secret type '%s' which does not have a deserialization method", type);
 	}
 
-	return secret_type.deserializer(deserializer, {scope, type, provider, name});
+	return deserialized_type.deserializer(deserializer, {scope, type, provider, name});
 }
 
 void SecretManager::RegisterSecretType(SecretType &type) {
 	lock_guard<mutex> lck(manager_lock);
-	auto lookup = secret_types.find(type.name);
-	if (lookup != secret_types.end()) {
-		throw InternalException("Attempted to register an already registered secret type: '%s'", type.name);
-	}
-	secret_types[type.name] = type;
+	RegisterSecretTypeInternal(type);
 }
 
 void SecretManager::RegisterSecretFunction(CreateSecretFunction function, OnCreateConflict on_conflict) {
 	unique_lock<mutex> lck(manager_lock);
-	auto lookup = secret_functions.find(function.secret_type);
-	if (lookup != secret_functions.end()) {
-		lookup->second.AddFunction(function, on_conflict);
-		return;
-	}
-	CreateSecretFunctionSet new_set(function.secret_type);
-	new_set.AddFunction(function, OnCreateConflict::ERROR_ON_CONFLICT);
-	secret_functions.insert({function.secret_type, new_set});
+	RegisterSecretFunctionInternal(std::move(function), on_conflict);
 }
 
 unique_ptr<SecretEntry> SecretManager::RegisterSecret(CatalogTransaction transaction,
@@ -407,11 +410,20 @@ SecretType SecretManager::LookupType(const string &type) {
 	return LookupTypeInternal(type);
 }
 
-SecretType SecretManager::LookupTypeInternal(const string &type) {
+void SecretManager::RegisterSecretTypeInternal(SecretType &type) {
+	auto lookup = secret_types.find(type.name);
+	if (lookup != secret_types.end()) {
+		throw InternalException("Attempted to register an already registered secret type: '%s'", type.name);
+	}
+	secret_types[type.name] = type;
+}
+
+bool SecretManager::TryLookupTypeInternal(const string &type, SecretType &type_out) {
 	unique_lock<mutex> lck(manager_lock);
 	auto lookup = secret_types.find(type);
 	if (lookup != secret_types.end()) {
-		return lookup->second;
+		type_out = lookup->second;
+		return true;
 	}
 
 	// Try autoloading
@@ -421,10 +433,30 @@ SecretType SecretManager::LookupTypeInternal(const string &type) {
 
 	lookup = secret_types.find(type);
 	if (lookup != secret_types.end()) {
-		return lookup->second;
+		type_out = lookup->second;
+		return true;
 	}
 
-	ThrowTypeNotFoundError(type);
+	return false;
+}
+
+SecretType SecretManager::LookupTypeInternal(const string &type) {
+	SecretType return_value;
+	if (!TryLookupTypeInternal(type, return_value)) {
+		ThrowTypeNotFoundError(type);
+	}
+	return return_value;
+}
+
+void SecretManager::RegisterSecretFunctionInternal(CreateSecretFunction function, OnCreateConflict on_conflict) {
+	auto lookup = secret_functions.find(function.secret_type);
+	if (lookup != secret_functions.end()) {
+		lookup->second.AddFunction(function, on_conflict);
+		return;
+	}
+	CreateSecretFunctionSet new_set(function.secret_type);
+	new_set.AddFunction(function, OnCreateConflict::ERROR_ON_CONFLICT);
+	secret_functions.insert({function.secret_type, new_set});
 }
 
 vector<SecretEntry> SecretManager::AllSecrets(CatalogTransaction transaction) {
@@ -465,12 +497,10 @@ bool SecretManager::PersistentSecretsEnabled() {
 }
 
 void SecretManager::SetDefaultStorage(const string &storage) {
-	ThrowOnSettingChangeIfInitialized();
 	config.default_persistent_storage = storage;
 }
 
 void SecretManager::ResetDefaultStorage() {
-	ThrowOnSettingChangeIfInitialized();
 	config.default_persistent_storage = SecretManager::LOCAL_FILE_STORAGE_NAME;
 }
 
@@ -517,16 +547,30 @@ void SecretManager::AutoloadExtensionForType(const string &type) {
 	ExtensionHelper::TryAutoloadFromEntry(*db, StringUtil::Lower(type), EXTENSION_SECRET_TYPES);
 }
 
-void SecretManager::ThrowTypeNotFoundError(const string &type) {
+void SecretManager::ThrowTypeNotFoundError(const string &type, const string &secret_path) {
 	auto entry = ExtensionHelper::FindExtensionInEntries(StringUtil::Lower(type), EXTENSION_SECRET_TYPES);
-	if (!entry.empty() && db) {
-		auto error_message = "Secret type '" + type + "' does not exist, but it exists in the " + entry + " extension.";
-		error_message =
-		    ExtensionHelper::AddExtensionInstallHintToErrorMsg(DBConfig::GetConfig(*db), error_message, entry);
+	string error_message;
 
-		throw InvalidInputException(error_message);
+	if (!entry.empty() && db) {
+		error_message = "Secret type '" + type + "' does not exist, but it exists in the " + entry + " extension.";
+		error_message = ExtensionHelper::AddExtensionInstallHintToErrorMsg(*db, error_message, entry);
+
+		if (!secret_path.empty()) {
+			error_message += "\n\nAlternatively, ";
+		}
+	} else {
+		error_message = StringUtil::Format("Secret type '%s' not found", type);
+
+		if (!secret_path.empty()) {
+			error_message += ", ";
+		}
 	}
-	throw InvalidInputException("Secret type '%s' not found", type);
+
+	if (!secret_path.empty()) {
+		error_message += StringUtil::Format("try removing the secret at path '%s'.", secret_path);
+	}
+
+	throw InvalidInputException(error_message);
 }
 
 void SecretManager::AutoloadExtensionForFunction(const string &type, const string &provider) {
@@ -541,8 +585,7 @@ void SecretManager::ThrowProviderNotFoundError(const string &type, const string
 		string error_message = was_default ? "Default secret provider" : "Secret provider";
 		error_message +=
 		    " '" + provider + "' for type '" + type + "' does not exist, but it exists in the " + entry + " extension.";
-		error_message =
-		    ExtensionHelper::AddExtensionInstallHintToErrorMsg(DBConfig::GetConfig(*db), error_message, entry);
+		error_message = ExtensionHelper::AddExtensionInstallHintToErrorMsg(*db, error_message, entry);
 
 		throw InvalidInputException(error_message);
 	}
@@ -577,15 +620,13 @@ DefaultSecretGenerator::DefaultSecretGenerator(Catalog &catalog, SecretManager &
     : DefaultGenerator(catalog), secret_manager(secret_manager), persistent_secrets(persistent_secrets) {
 }
 
-unique_ptr<CatalogEntry> DefaultSecretGenerator::CreateDefaultEntry(ClientContext &context, const string &entry_name) {
-
+unique_ptr<CatalogEntry> DefaultSecretGenerator::CreateDefaultEntryInternal(const string &entry_name) {
 	auto secret_lu = persistent_secrets.find(entry_name);
 	if (secret_lu == persistent_secrets.end()) {
 		return nullptr;
 	}
 
 	LocalFileSystem fs;
-	auto &catalog = Catalog::GetSystemCatalog(context);
 
 	string base_secret_path = secret_manager.PersistentSecretPath();
 	string secret_path = fs.JoinPath(base_secret_path, entry_name + ".duckdb_secret");
@@ -604,7 +645,7 @@ unique_ptr<CatalogEntry> DefaultSecretGenerator::CreateDefaultEntry(ClientContex
 			BinaryDeserializer deserializer(file_reader);
 
 			deserializer.Begin();
-			auto deserialized_secret = secret_manager.DeserializeSecret(deserializer);
+			auto deserialized_secret = secret_manager.DeserializeSecret(deserializer, secret_path);
 			deserializer.End();
 
 			auto entry = make_uniq<SecretCatalogEntry>(std::move(deserialized_secret), catalog);
@@ -640,6 +681,15 @@ unique_ptr<CatalogEntry> DefaultSecretGenerator::CreateDefaultEntry(ClientContex
 	                             entry_name, secret_path);
 }
 
+unique_ptr<CatalogEntry> DefaultSecretGenerator::CreateDefaultEntry(CatalogTransaction transaction,
+                                                                    const string &entry_name) {
+	return CreateDefaultEntryInternal(entry_name);
+}
+
+unique_ptr<CatalogEntry> DefaultSecretGenerator::CreateDefaultEntry(ClientContext &context, const string &entry_name) {
+	return CreateDefaultEntryInternal(entry_name);
+}
+
 vector<string> DefaultSecretGenerator::GetDefaultEntries() {
 	vector<string> ret;
 
