@@ -47,6 +47,12 @@ unique_ptr<LogicalOperator> FilterPushdown::PushdownInnerJoin(unique_ptr<Logical
 
 	// turn the inner join into a cross product
 	auto cross_product = make_uniq<LogicalCrossProduct>(std::move(op->children[0]), std::move(op->children[1]));
+
+	// preserve the estimated cardinality of the operator
+	if (op->has_estimated_cardinality) {
+		cross_product->SetEstimatedCardinality(op->estimated_cardinality);
+	}
+
 	// then push down cross product
 	return PushdownCrossProduct(std::move(cross_product));
 }
