@@ -1,6 +1,7 @@
 #include "duckdb/parallel/task.hpp"
 #include "duckdb/execution/executor.hpp"
 #include "duckdb/main/client_context.hpp"
+#include "duckdb/parallel/thread_context.hpp"
 
 namespace duckdb {
 
@@ -9,11 +10,16 @@ ExecutorTask::ExecutorTask(Executor &executor_p, shared_ptr<Event> event_p)
 	executor.RegisterTask();
 }
 
-ExecutorTask::ExecutorTask(ClientContext &context, shared_ptr<Event> event_p)
-    : ExecutorTask(Executor::Get(context), std::move(event_p)) {
+ExecutorTask::ExecutorTask(ClientContext &context, shared_ptr<Event> event_p, const PhysicalOperator &op_p)
+    : executor(Executor::Get(context)), event(std::move(event_p)), op(&op_p) {
+	thread_context = make_uniq<ThreadContext>(context);
+	executor.RegisterTask();
 }
 
 ExecutorTask::~ExecutorTask() {
+	if (thread_context) {
+		executor.Flush(*thread_context);
+	}
 	executor.UnregisterTask();
 }
 
@@ -29,7 +35,14 @@ void ExecutorTask::Reschedule() {
 
 TaskExecutionResult ExecutorTask::Execute(TaskExecutionMode mode) {
 	try {
-		return ExecuteTask(mode);
+		if (thread_context) {
+			thread_context->profiler.StartOperator(op);
+			auto result = ExecuteTask(mode);
+			thread_context->profiler.EndOperator(nullptr);
+			return result;
+		} else {
+			return ExecuteTask(mode);
+		}
 	} catch (std::exception &ex) {
 		executor.PushError(ErrorData(ex));
 	} catch (...) { // LCOV_EXCL_START
