@@ -87,9 +87,7 @@ BindResult ExpressionBinder::BindExpression(FunctionExpression &function, idx_t
 
 	switch (func->type) {
 	case CatalogType::SCALAR_FUNCTION_ENTRY: {
-		// scalar function
-		if (IsLambdaFunction(function)) {
-			// special case
+		if (function.IsLambdaFunction()) {
 			return TryBindLambdaOrJson(function, depth, *func);
 		}
 		return BindFunction(function, func->Cast<ScalarFunctionCatalogEntry>(), depth);
@@ -265,11 +263,11 @@ BindResult ExpressionBinder::BindLambdaFunction(FunctionExpression &function, Sc
 
 BindResult ExpressionBinder::BindAggregate(FunctionExpression &expr, AggregateFunctionCatalogEntry &function,
                                            idx_t depth) {
-	return BindResult(BinderException(expr, UnsupportedAggregateMessage()));
+	return BindUnsupportedExpression(expr, depth, UnsupportedAggregateMessage());
 }
 
 BindResult ExpressionBinder::BindUnnest(FunctionExpression &expr, idx_t depth, bool root_expression) {
-	return BindResult(BinderException(expr, UnsupportedUnnestMessage()));
+	return BindUnsupportedExpression(expr, depth, UnsupportedUnnestMessage());
 }
 
 string ExpressionBinder::UnsupportedAggregateMessage() {
