@@ -10,13 +10,13 @@ BindResult RelationBinder::BindExpression(unique_ptr<ParsedExpression> &expr_ptr
 	auto &expr = *expr_ptr;
 	switch (expr.expression_class) {
 	case ExpressionClass::AGGREGATE:
-		return BindResult("aggregate functions are not allowed in " + op);
+		return BindResult(BinderException::Unsupported(expr, "aggregate functions are not allowed in " + op));
 	case ExpressionClass::DEFAULT:
-		return BindResult(op + " cannot contain DEFAULT clause");
+		return BindResult(BinderException::Unsupported(expr, op + " cannot contain DEFAULT clause"));
 	case ExpressionClass::SUBQUERY:
-		return BindResult("subqueries are not allowed in " + op);
+		return BindResult(BinderException::Unsupported(expr, "subqueries are not allowed in " + op));
 	case ExpressionClass::WINDOW:
-		return BindResult("window functions are not allowed in " + op);
+		return BindResult(BinderException::Unsupported(expr, "window functions are not allowed in " + op));
 	default:
 		return ExpressionBinder::BindExpression(expr_ptr, depth);
 	}
