@@ -4,10 +4,10 @@ namespace duckdb {
 
 idx_t LogicalTopN::EstimateCardinality(ClientContext &context) {
 	auto child_cardinality = LogicalOperator::EstimateCardinality(context);
-	if (limit >= 0 && child_cardinality < idx_t(limit)) {
-		return limit;
+	if (child_cardinality < limit) {
+		return child_cardinality;
 	}
-	return child_cardinality;
+	return limit;
 }
 
 } // namespace duckdb
