@@ -4,12 +4,17 @@
 #include "duckdb/catalog/catalog_entry/table_catalog_entry.hpp"
 #include "duckdb/catalog/catalog_entry/type_catalog_entry.hpp"
 #include "duckdb/catalog/catalog_entry/view_catalog_entry.hpp"
+#include "duckdb/common/checksum.hpp"
 #include "duckdb/common/printer.hpp"
 #include "duckdb/common/serializer/binary_deserializer.hpp"
 #include "duckdb/common/serializer/buffered_file_reader.hpp"
+#include "duckdb/common/serializer/memory_stream.hpp"
 #include "duckdb/common/string_util.hpp"
+#include "duckdb/execution/index/art/art.hpp"
+#include "duckdb/execution/index/index_type_set.hpp"
 #include "duckdb/main/attached_database.hpp"
 #include "duckdb/main/client_context.hpp"
+#include "duckdb/main/config.hpp"
 #include "duckdb/main/connection.hpp"
 #include "duckdb/main/database.hpp"
 #include "duckdb/parser/parsed_data/alter_table_info.hpp"
@@ -20,14 +25,10 @@
 #include "duckdb/planner/expression_binder/index_binder.hpp"
 #include "duckdb/planner/parsed_data/bound_create_table_info.hpp"
 #include "duckdb/storage/storage_manager.hpp"
-#include "duckdb/storage/write_ahead_log.hpp"
-#include "duckdb/common/serializer/memory_stream.hpp"
-#include "duckdb/common/checksum.hpp"
-#include "duckdb/execution/index/index_type_set.hpp"
-#include "duckdb/execution/index/art/art.hpp"
-#include "duckdb/main/config.hpp"
 #include "duckdb/storage/table/delete_state.hpp"
+#include "duckdb/storage/write_ahead_log.hpp"
 #include "duckdb/transaction/meta_transaction.hpp"
+#include "duckdb/storage/table/column_data.hpp"
 
 namespace duckdb {
 
@@ -86,10 +87,9 @@ public:
 		// compute and verify the checksum
 		auto computed_checksum = Checksum(buffer.get(), size);
 		if (stored_checksum != computed_checksum) {
-			throw SerializationException(
-			    "Corrupt WAL file: entry at byte position %llu computed checksum %llu does not match "
-			    "stored checksum %llu",
-			    offset, computed_checksum, stored_checksum);
+			throw IOException("Corrupt WAL file: entry at byte position %llu computed checksum %llu does not match "
+			                  "stored checksum %llu",
+			                  offset, computed_checksum, stored_checksum);
 		}
 		return WriteAheadLogDeserializer(state_p, std::move(buffer), size, deserialize_only);
 	}
@@ -143,6 +143,7 @@ protected:
 
 	void ReplayUseTable();
 	void ReplayInsert();
+	void ReplayRowGroupData();
 	void ReplayDelete();
 	void ReplayUpdate();
 	void ReplayCheckpoint();
@@ -310,6 +311,9 @@ void WriteAheadLogDeserializer::ReplayEntry(WALType entry_type) {
 	case WALType::INSERT_TUPLE:
 		ReplayInsert();
 		break;
+	case WALType::ROW_GROUP_DATA:
+		ReplayRowGroupData();
+		break;
 	case WALType::DELETE_TUPLE:
 		ReplayDelete();
 		break;
@@ -551,7 +555,7 @@ void WriteAheadLogDeserializer::ReplayCreateIndex() {
 
 			// read the data into a buffer handle
 			shared_ptr<BlockHandle> block_handle;
-			buffer_manager.Allocate(MemoryTag::ART_INDEX, Storage::BLOCK_SIZE, false, &block_handle);
+			buffer_manager.Allocate(MemoryTag::ART_INDEX, block_manager->GetBlockSize(), false, &block_handle);
 			auto buffer_handle = buffer_manager.Pin(block_handle);
 			auto data_ptr = buffer_handle.Ptr();
 
@@ -581,20 +585,7 @@ void WriteAheadLogDeserializer::ReplayCreateIndex() {
 
 	// create the index in the catalog
 	auto &table = catalog.GetEntry<TableCatalogEntry>(context, create_info->schema, info.table).Cast<DuckTableEntry>();
-	auto &index = catalog.CreateIndex(context, info)->Cast<DuckIndexEntry>();
-	index.info = make_shared_ptr<IndexDataTableInfo>(table.GetStorage().GetDataTableInfo(), index.name);
-
-	// insert the parsed expressions into the index so that we can (de)serialize them during consecutive checkpoints
-	for (auto &parsed_expr : info.parsed_expressions) {
-		index.parsed_expressions.push_back(parsed_expr->Copy());
-	}
-
-	// obtain the parsed expressions of the ART from the index metadata
-	vector<unique_ptr<ParsedExpression>> parsed_expressions;
-	for (auto &parsed_expr : info.parsed_expressions) {
-		parsed_expressions.push_back(parsed_expr->Copy());
-	}
-	D_ASSERT(!parsed_expressions.empty());
+	auto &index = table.schema.CreateIndex(context, info, table)->Cast<DuckIndexEntry>();
 
 	// add the table to the bind context to bind the parsed expressions
 	auto binder = Binder::CreateBinder(context);
@@ -612,9 +603,10 @@ void WriteAheadLogDeserializer::ReplayCreateIndex() {
 
 	// bind the parsed expressions to create unbound expressions
 	vector<unique_ptr<Expression>> unbound_expressions;
-	unbound_expressions.reserve(parsed_expressions.size());
-	for (auto &expr : parsed_expressions) {
-		unbound_expressions.push_back(idx_binder.Bind(expr));
+	unbound_expressions.reserve(index.parsed_expressions.size());
+	for (auto &expr : index.parsed_expressions) {
+		auto copy = expr->Copy();
+		unbound_expressions.push_back(idx_binder.Bind(copy));
 	}
 
 	auto &data_table = table.GetStorage();
@@ -666,6 +658,54 @@ void WriteAheadLogDeserializer::ReplayInsert() {
 	state.current_table->GetStorage().LocalAppend(*state.current_table, context, chunk, bound_constraints);
 }
 
+static void MarkBlocksAsUsed(BlockManager &manager, const PersistentColumnData &col_data) {
+	for (auto &pointer : col_data.pointers) {
+		auto block_id = pointer.block_pointer.block_id;
+		if (block_id != INVALID_BLOCK) {
+			manager.MarkBlockAsUsed(block_id);
+		}
+		if (pointer.segment_state) {
+			for (auto &block : pointer.segment_state->blocks) {
+				manager.MarkBlockAsUsed(block);
+			}
+		}
+	}
+	for (auto &child_column : col_data.child_columns) {
+		MarkBlocksAsUsed(manager, child_column);
+	}
+}
+
+void WriteAheadLogDeserializer::ReplayRowGroupData() {
+	auto &block_manager = db.GetStorageManager().GetBlockManager();
+	PersistentCollectionData data;
+	deserializer.Set<DatabaseInstance &>(db.GetDatabase());
+	CompressionInfo compression_info(block_manager.GetBlockSize());
+	deserializer.Set<const CompressionInfo &>(compression_info);
+	deserializer.ReadProperty(101, "row_group_data", data);
+	deserializer.Unset<const CompressionInfo>();
+	deserializer.Unset<DatabaseInstance>();
+	if (DeserializeOnly()) {
+		// label blocks in data as used - they will be used after the WAL replay is finished
+		// we need to do this during the deserialization phase to ensure the blocks will not be overwritten
+		// by previous deserialization steps
+		for (auto &group : data.row_group_data) {
+			for (auto &col_data : group.column_data) {
+				MarkBlocksAsUsed(block_manager, col_data);
+			}
+		}
+		return;
+	}
+	if (!state.current_table) {
+		throw InternalException("Corrupt WAL: insert without table");
+	}
+	auto &storage = state.current_table->GetStorage();
+	auto &table_info = storage.GetDataTableInfo();
+	RowGroupCollection new_row_groups(table_info, block_manager, storage.GetTypes(), 0);
+	new_row_groups.Initialize(data);
+	TableIndexList index_list;
+	storage.MergeStorage(new_row_groups, index_list, nullptr);
+}
+
 void WriteAheadLogDeserializer::ReplayDelete() {
 	DataChunk chunk;
 	deserializer.ReadObject(101, "chunk", [&](Deserializer &object) { chunk.Deserialize(object); });
