@@ -12,8 +12,6 @@
 
 #include "src/execution/join_hashtable.cpp"
 
-#include "src/execution/partitionable_hashtable.cpp"
-
 #include "src/execution/perfect_aggregate_hashtable.cpp"
 
 #include "src/execution/physical_operator.cpp"
@@ -24,5 +22,7 @@
 
 #include "src/execution/reservoir_sample.cpp"
 
+#include "src/execution/window_executor.cpp"
+
 #include "src/execution/window_segment_tree.cpp"
 
