@@ -10,8 +10,12 @@
 
 #include "src/parser/transform/statement/transform_create_function.cpp"
 
+#include "src/parser/transform/statement/transform_comment_on.cpp"
+
 #include "src/parser/transform/statement/transform_copy.cpp"
 
+#include "src/parser/transform/statement/transform_copy_database.cpp"
+
 #include "src/parser/transform/statement/transform_create_table_as.cpp"
 
 #include "src/parser/transform/statement/transform_create_index.cpp"
@@ -42,6 +46,8 @@
 
 #include "src/parser/transform/statement/transform_rename.cpp"
 
+#include "src/parser/transform/statement/transform_secret.cpp"
+
 #include "src/parser/transform/statement/transform_select.cpp"
 
 #include "src/parser/transform/statement/transform_select_node.cpp"
