@@ -105,20 +105,49 @@ class TestDateTimeTimeStamp(object):
     def test_timestamp_timezone_positive_extreme(self, pandas, duckdb_cursor):
         duckdb_time = duckdb_cursor.sql(
             """
-            SELECT timestamp '2021-12-31 23:00:00' AT TIME ZONE 'kea_CV' as "0"
+            SELECT timestamp '2021-12-31 23:00:00' AT TIME ZONE 'Etc/GMT-14' as "0"
         """
         ).df()
 
-        # 'kea_CV' is 20 hours ahead of UTC
-        offset = datetime.timedelta(hours=20)
+        offset = datetime.timedelta(hours=14)
         timezone = datetime.timezone(offset)
 
         df_in = pandas.DataFrame(
             {
                 0: pandas.Series(
-                    data=[datetime.datetime(year=2022, month=1, day=1, hour=19, tzinfo=timezone)], dtype='object'
+                    data=[datetime.datetime(year=2021, month=12, day=31, hour=23, tzinfo=timezone)], dtype='object'
                 )
             }
         )
         df_out = duckdb_cursor.sql("""select * from df_in""").df()
         pandas.testing.assert_frame_equal(df_out, duckdb_time)
+
+    @pytest.mark.skipif(
+        Version(pd.__version__) < Version('2.0.2'), reason="pandas < 2.0.2 does not properly convert timezones"
+    )
+    @pytest.mark.parametrize('unit', ['ms', 'ns', 's'])
+    def test_timestamp_timezone_coverage(self, unit, duckdb_cursor):
+        pd = pytest.importorskip("pandas")
+        ts_df = pd.DataFrame(
+            {'ts': pd.Series(data=[pd.Timestamp(datetime.datetime(1990, 12, 21))], dtype=f'datetime64[{unit}]')}
+        )
+        usecond_df = pd.DataFrame(
+            {'ts': pd.Series(data=[pd.Timestamp(datetime.datetime(1990, 12, 21))], dtype='datetime64[us]')}
+        )
+
+        query = """
+            select
+                cast(ts as timestamptz) as tstz
+            from {}
+        """
+
+        duckdb_cursor.sql("set TimeZone = 'UTC'")
+        utc_usecond = duckdb_cursor.sql(query.format('usecond_df')).df()
+        utc_other = duckdb_cursor.sql(query.format('ts_df')).df()
+
+        duckdb_cursor.sql("set TimeZone = 'America/Los_Angeles'")
+        us_usecond = duckdb_cursor.sql(query.format('usecond_df')).df()
+        us_other = duckdb_cursor.sql(query.format('ts_df')).df()
+
+        pd.testing.assert_frame_equal(utc_usecond, utc_other)
+        pd.testing.assert_frame_equal(us_usecond, us_other)
